@@ -172,6 +172,8 @@ function _make_targetinfo(mode, arch, target)
         end
     end
     for k, v in table.orderpairs(addrunenvs) do
+        -- https://github.com/xmake-io/xmake/issues/3391
+        v = table.unique(v)
         if k:upper() == "PATH" then
             runenvs[k] = _make_dirs(v) .. ";$([System.Environment]::GetEnvironmentVariable('" .. k .. "'))"
         else
@@ -263,8 +265,15 @@ function _make_vsinfo_archs()
             end
             if default_archs then
                 default_archs = hashset.from(table.wrap(default_archs))
-                default_archs:remove("arm64")
-                vsinfo_archs = default_archs:to_array()
+                -- just generate single arch by default to avoid some fails for installing packages.
+                -- @see https://github.com/xmake-io/xmake/issues/3268
+                local arch = config.arch()
+                if default_archs:has(arch) then
+                    vsinfo_archs = { arch }
+                else
+                    default_archs:remove("arm64")
+                    vsinfo_archs = default_archs:to_array()
+                end
             end
         end
     end
@@ -279,7 +288,7 @@ function _make_vsinfo_groups()
     local group_deps = {}
     for targetname, target in table.orderpairs(project.targets()) do
         local group_path = target:get("group")
-        if group_path then
+        if group_path and #(group_path:trim()) > 0 then
             local group_name = path.filename(group_path)
             local group_names = path.split(group_path)
             for idx, name in ipairs(group_names) do
@@ -385,7 +394,11 @@ function _make_filter(filepath, target, vcxprojdir)
                         filter = path.normalize(filegroup)
                     else
                         -- file tree mode (default)
-                        filter = path.normalize(path.join(filegroup, path.directory(fileitem)))
+                        if filegroup ~= "" then
+                            filter = path.normalize(path.join(filegroup, path.directory(fileitem)))
+                        else
+                            filter = path.normalize(path.directory(fileitem))
+                        end
                     end
                     if filter and filter == '.' then
                         filter = nil
